import os
import sqlite3
import sys
import argparse
from search_fs import DEFAULT_DB_FILE, DIRECTORY_TYPE, FILE_TYPE

SUFFIXES = ['B', 'KB', 'MB', 'GB', 'TB']


def where_clause(ns):
    name = ns.name
    type = ns.type
    dirs = ns.directories
    strict_dir = ns.strict_dir
    size = ns.size
    args = []
    queries = []
    if dirs:
        if strict_dir:
            dirs = [os.path.abspath(d) for d in dirs]
            parent_query = ' OR '.join('parent = ?' for i in range(len(dirs)))
        else:
            dirs = [os.path.abspath(d) + '%' for d in dirs]
            parent_query = ' OR '.join('parent like ?' for i in range(len(dirs)))
        queries.append('(' + parent_query + ')')
        args.extend(dirs)
    if name:
        name = name.replace('*', '%')
        args.append(name)
        queries.append('name like ?')
    if type:
        queries.append('type = ?')
        if type == 'f':
            args.append(FILE_TYPE)
        elif type == 'd':
            args.append(DIRECTORY_TYPE)
    if size is not None:
        size = size.upper()
        if not (size.startswith('+') or size.startswith('-')):
            size = '+' + size
        if not size.endswith('B'):
            size = size + 'B'
        num = float(size[1:-2])
        suffix = size[-2:]
        try:
            num = num * (1024 ** SUFFIXES.index(suffix))
        except ValueError:
            print('Unknown suffix for {}'.format(ns.size))
            sys.exit(2)
        if size.startswith('+'):
            queries.append('size >= ?')
        elif size.startswith('-'):
            queries.append('size <= ?')
        args.append(num)

    return ' AND '.join(queries), args


def search(ns):
    end_char = '\0' if ns.zero else '\n'
    with sqlite3.connect(ns.database) as conn:
        where, args = where_clause(ns)
        query = 'select path from files where ' + where
        c = conn.cursor()
        c.execute(query, args)
        for row in c:
            print(row[0], end=end_char)


def main():
    parser = argparse.ArgumentParser(description='Search the filesystem search index')
    parser.add_argument('--database', '--db', default=DEFAULT_DB_FILE,
                        help='The database file to use. Defaults to {}'.format(DEFAULT_DB_FILE))
    parser.add_argument('--name', '-n', help='Search by name')
    parser.add_argument('--type', '-t', choices=['f', 'd'], help='Search by type')
    parser.add_argument('--size', '-s', help='Search by size. Prepend with + for greater, or - for less.')
    parser.add_argument('-0', dest='zero', action='store_const', const=True, default=False,
                        help='Output results separated by null byte (for xargs -0)')
    parser.add_argument('--strict-dir', '-d', dest='strict_dir', action='store_const', const=True, default=False,
                        help='Match only exact directories instead of walking the tree')
    parser.add_argument('directories', metavar='dir', nargs='*')

    ns = parser.parse_args()

    if not (ns.name or ns.type):
        print('Please specify name or type for searching')
        sys.exit(1)
    search(ns)



import os
import sys
import sqlite3
from itertools import islice
import argparse
from search_fs import DEFAULT_DB_FILE, DIRECTORY_TYPE, FILE_TYPE
import time


def split_every(n, iterable):
    i = iter(iterable)
    piece = list(islice(i, n))
    while piece:
        yield piece
        piece = list(islice(i, n))


def walk(input_dirs):
    for input_dir in input_dirs:
        for dir, dirnames, filenames in os.walk(input_dir):
            for filename in filenames:
                path = os.path.join(dir, filename)
                size = os.stat(path, follow_symlinks=False).st_size
                yield path, dir, filename, FILE_TYPE, size
            for dirname in dirnames:
                path = os.path.join(dir, dirname)
                yield path, dir, dirname, DIRECTORY_TYPE, 0


def create_db(db_file, input_dirs):
    print('Indexing {} directories'.format(len(input_dirs)))
    temp_db = db_file + '.temp'
    if os.path.exists(temp_db):
        os.remove(temp_db)
    start = time.time()
    with sqlite3.connect(temp_db) as conn:
        conn.execute(
            'create table files(path text unique not null, parent text not null, name text not null, type integer not null, size integer not null)')
        file_count = 0
        results = walk(input_dirs)
        for split in split_every(10000, results):
            file_count += len(split)
            print('Processed {} files'.format(file_count))
            conn.executemany('insert into files(path, parent, name, type, size) values (?,?,?,?,?)', split)
        conn.execute('create index parent_idx on files(parent)')
        conn.execute('create index name_idx on files(name)')
        conn.execute('create index size_idx on files(size)')
    os.replace(src=temp_db, dst=db_file)
    end = time.time()
    print('Completed in {0:.2f}s'.format(end - start))


def main():
    parser = argparse.ArgumentParser(description='Create the filesystem search index')
    parser.add_argument('--output', '-o', default=DEFAULT_DB_FILE,
                        help='The output database file. Defaults to {}'.format(DEFAULT_DB_FILE))
    parser.add_argument('--dirs', metavar='FILE', help='File containing a list of directories to index')
    parser.add_argument('directories', metavar='dir', nargs='*')

    ns = parser.parse_args()
    directories = ns.directories.copy()
    if ns.dirs:
        with open(ns.dirs) as f:
            for line in f.readlines():
                if line and not line.startswith('#'):
                    directories.append(line.strip())
    if not directories:
        print('You must specify at least one directory to index')
        sys.exit(1)
    create_db(ns.output, directories)



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(name='search-fs',
      version='0.0.1',
      description='Scripts for managing media',
      long_description=long_description,
      long_description_content_type="text/markdown",
      url='https://github.com/raydouglass/search-fs',
      author='Ray Douglass',
      author_email='ray@raydouglass.com',
      license='Apache',
      packages=find_packages(),
      zip_safe=True,
      entry_points={
          'console_scripts': [
              'search-fs=search_fs.search:main',
              'create-search-fs=search_fs.create_db:main'
          ],
      },
      install_requires=[],
      python_requires='>=3.6',
      classifiers=[
          "Programming Language :: Python :: 3",
          "License :: OSI Approved :: Apache Software License",
          "Operating System :: OS Independent",
      ]
      )
